package view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import javax.swing.JPanel;
import model.*;

public class GamePanel extends JPanel{
    private final int scale;
    private final int width, height;
    private final GameBoard board;
    
    private final int halfSize, centerOffset;
    private final int smallSize, smallOffset;
    private final int mediumSize, mediumOffset;
    private final int largeSize, largeOffset;
    private final BufferedImage level;

    public GamePanel() {
        this(null);
    }
        
    public GamePanel(GameBoard board){
        if (board != null){
            this.board = board;
            scale = board.scale;
            width = board.getWidth();
            height = board.getHeight();
        } else { // Designer couldn't load
            this.board = null;
            width = 10;
            height = 10;
            scale = 32;
        }
        halfSize = centerOffset = scale / 2;
        smallSize = scale / 4;
        mediumSize = scale;
        largeSize = (int)(scale * 1.5);
        smallOffset = (mediumSize - smallSize) / 2;
        mediumOffset = 0;
        largeOffset = (mediumSize - largeSize) / 2;
        level = new BufferedImage(width * scale, height * scale, BufferedImage.TYPE_INT_RGB);
        if (board != null) drawWalls(level.getGraphics());
        
        Dimension d = new Dimension(width * scale, height * scale);
        setSize(d);
        setPreferredSize(d);
        setMinimumSize(d);
        setMaximumSize(d);
    }

    @Override
    protected void paintComponent(Graphics grphcs) {
        if (board == null) return;
        Graphics2D g = (Graphics2D)grphcs;

        g.drawImage(level, 0, 0, null);
        Item[][] table = board.getTable();
        
        for (int y = 0, yTop = 0; y < height; y++, yTop += scale){
            for (int x = 0, xLeft = 0; x < width; x++, xLeft += scale){
                Item i = table[y][x];
                if (i == Item.FOOD){
                    g.setColor(new Color(255, 192, 160));
                    g.fillOval(xLeft + smallOffset, yTop + smallOffset, smallSize, smallSize);
                } else if (i == Item.POWER){
                    g.setColor(new Color(255, 192, 160));
                    g.fillOval(xLeft + mediumOffset, yTop + mediumOffset, mediumSize, mediumSize);
                } 
            }
        }
        for (int i = 0; i < board.numPlayers(); i++){
            drawPlayer(grphcs, board.getPlayer(i));
        }
    }
    
    private void drawPlayer(Graphics gr, Player p){
        Graphics2D g = (Graphics2D)gr;
        int x = p.smoothPos.x;
        int y = p.smoothPos.y;
        g.setColor(p.color);
        g.fillArc(x + largeOffset, y + largeOffset, largeSize, largeSize, p.faceDir.angle + p.getMouth(), 360 - p.getMouth() * 2);
        g.setColor(Color.BLACK);
        if (p.faceDir == Direction.RIGHT){
            g.fillOval(x + centerOffset, y + mediumOffset, smallSize, smallSize);
        } else if (p.faceDir == Direction.UP){
            g.fillOval(x + mediumOffset, y + centerOffset - smallSize, smallSize, smallSize);
        } else if (p.faceDir == Direction.LEFT){
            g.fillOval(x + centerOffset - smallSize, y + mediumOffset, smallSize, smallSize);
        } else if (p.faceDir == Direction.DOWN){
            g.fillOval(x + centerOffset + smallSize, y + centerOffset, smallSize, smallSize);
        }
    }
    
    private void drawWalls(Graphics gr){
        Graphics2D g = (Graphics2D)gr;
        Item table[][] = board.getTable();

        g.setColor(Color.BLACK);
        g.fillRect(0, 0, width * scale, height * scale);
        g.setColor(new Color(0, 128, 255));
        
        for (int y = 0, yTop = 0; y < height; y++, yTop += scale){
            for (int x = 0, xLeft = 0; x < width; x++, xLeft += scale){
                if (table[y][x] != Item.WALL) continue;
                
                g.fillOval(xLeft + smallOffset, yTop + smallOffset, smallSize, smallSize);

                boolean leftWall = (x > 0 && table[y][x - 1] == Item.WALL);
                boolean topWall = (y > 0 && table[y - 1][x] == Item.WALL);
                boolean rightWall = (x < width - 1 && table[y][x + 1] == Item.WALL);
                boolean bottomWall = (y < height - 1 && table[y + 1][x] == Item.WALL);

                boolean topLeftFree = (x > 0 && y > 0 && table[y - 1][x - 1] != Item.WALL);
                boolean topRightFree = (x < width - 1 && y > 0 && table[y - 1][x + 1] != Item.WALL);
                boolean bottomLeftFree = (x > 0 && y < height - 1 && table[y + 1][x - 1] != Item.WALL);
                boolean bottomRightFree = (x < width - 1 && y < height - 1 && table[y + 1][x + 1] != Item.WALL);

                if (topWall && (topLeftFree || topRightFree || !leftWall || !rightWall)){
                    g.fillRect(xLeft + smallOffset, yTop, smallSize, halfSize);
                }
                if (bottomWall && (bottomLeftFree || bottomRightFree || !leftWall || !rightWall)){
                    g.fillRect(xLeft + smallOffset, yTop + centerOffset, smallSize, halfSize);
                }
                if (leftWall && (bottomLeftFree || topLeftFree || !bottomWall || !topWall)){
                    g.fillRect(xLeft, yTop + smallOffset, halfSize, smallSize);
                }
                if (rightWall && (bottomRightFree || topRightFree || !bottomWall || !topWall)){
                    g.fillRect(xLeft + centerOffset, yTop + smallOffset, halfSize, smallSize);
                }
            }
        }
    }
}
